/****************************************************************************
** Meta object code from reading C++ file 'dsysinfo.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/global/dsysinfo.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dsysinfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Core__DSysInfo_t {
    QByteArrayData data[52];
    char stringdata0[381];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Core__DSysInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Core__DSysInfo_t qt_meta_stringdata_Dtk__Core__DSysInfo = {
    {
QT_MOC_LITERAL(0, 0, 19), // "Dtk::Core::DSysInfo"
QT_MOC_LITERAL(1, 20, 11), // "ProductType"
QT_MOC_LITERAL(2, 32, 11), // "UnknownType"
QT_MOC_LITERAL(3, 44, 6), // "Deepin"
QT_MOC_LITERAL(4, 51, 9), // "ArchLinux"
QT_MOC_LITERAL(5, 61, 6), // "CentOS"
QT_MOC_LITERAL(6, 68, 6), // "Debian"
QT_MOC_LITERAL(7, 75, 6), // "Fedora"
QT_MOC_LITERAL(8, 82, 9), // "LinuxMint"
QT_MOC_LITERAL(9, 92, 7), // "Manjaro"
QT_MOC_LITERAL(10, 100, 8), // "openSUSE"
QT_MOC_LITERAL(11, 109, 10), // "SailfishOS"
QT_MOC_LITERAL(12, 120, 6), // "Ubuntu"
QT_MOC_LITERAL(13, 127, 3), // "Uos"
QT_MOC_LITERAL(14, 131, 6), // "Gentoo"
QT_MOC_LITERAL(15, 138, 5), // "NixOS"
QT_MOC_LITERAL(16, 144, 4), // "Arch"
QT_MOC_LITERAL(17, 149, 5), // "ARM64"
QT_MOC_LITERAL(18, 155, 8), // "ARM64_BE"
QT_MOC_LITERAL(19, 164, 3), // "ARM"
QT_MOC_LITERAL(20, 168, 6), // "ARM_BE"
QT_MOC_LITERAL(21, 175, 5), // "ALPHA"
QT_MOC_LITERAL(22, 181, 5), // "SW_64"
QT_MOC_LITERAL(23, 187, 3), // "ARC"
QT_MOC_LITERAL(24, 191, 6), // "ARC_BE"
QT_MOC_LITERAL(25, 198, 4), // "CRIS"
QT_MOC_LITERAL(26, 203, 6), // "X86_64"
QT_MOC_LITERAL(27, 210, 3), // "X86"
QT_MOC_LITERAL(28, 214, 4), // "IA64"
QT_MOC_LITERAL(29, 219, 11), // "LOONGARCH64"
QT_MOC_LITERAL(30, 231, 4), // "M68K"
QT_MOC_LITERAL(31, 236, 9), // "MIPS64_LE"
QT_MOC_LITERAL(32, 246, 6), // "MIPS64"
QT_MOC_LITERAL(33, 253, 7), // "MIPS_LE"
QT_MOC_LITERAL(34, 261, 4), // "MIPS"
QT_MOC_LITERAL(35, 266, 5), // "NIOS2"
QT_MOC_LITERAL(36, 272, 8), // "PARISC64"
QT_MOC_LITERAL(37, 281, 6), // "PARISC"
QT_MOC_LITERAL(38, 288, 8), // "PPC64_LE"
QT_MOC_LITERAL(39, 297, 5), // "PPC64"
QT_MOC_LITERAL(40, 303, 3), // "PPC"
QT_MOC_LITERAL(41, 307, 6), // "PPC_LE"
QT_MOC_LITERAL(42, 314, 7), // "RISCV32"
QT_MOC_LITERAL(43, 322, 7), // "RISCV64"
QT_MOC_LITERAL(44, 330, 5), // "S390X"
QT_MOC_LITERAL(45, 336, 4), // "S390"
QT_MOC_LITERAL(46, 341, 4), // "SH64"
QT_MOC_LITERAL(47, 346, 2), // "SH"
QT_MOC_LITERAL(48, 349, 7), // "SPARC64"
QT_MOC_LITERAL(49, 357, 5), // "SPARC"
QT_MOC_LITERAL(50, 363, 6), // "TILEGX"
QT_MOC_LITERAL(51, 370, 10) // "NUM_ARCHES"

    },
    "Dtk::Core::DSysInfo\0ProductType\0"
    "UnknownType\0Deepin\0ArchLinux\0CentOS\0"
    "Debian\0Fedora\0LinuxMint\0Manjaro\0"
    "openSUSE\0SailfishOS\0Ubuntu\0Uos\0Gentoo\0"
    "NixOS\0Arch\0ARM64\0ARM64_BE\0ARM\0ARM_BE\0"
    "ALPHA\0SW_64\0ARC\0ARC_BE\0CRIS\0X86_64\0"
    "X86\0IA64\0LOONGARCH64\0M68K\0MIPS64_LE\0"
    "MIPS64\0MIPS_LE\0MIPS\0NIOS2\0PARISC64\0"
    "PARISC\0PPC64_LE\0PPC64\0PPC\0PPC_LE\0"
    "RISCV32\0RISCV64\0S390X\0S390\0SH64\0SH\0"
    "SPARC64\0SPARC\0TILEGX\0NUM_ARCHES"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Core__DSysInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       2,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,   14,   24,
      16,   16, 0x0,   35,   52,

 // enum data: key, value
       2, uint(Dtk::Core::DSysInfo::UnknownType),
       3, uint(Dtk::Core::DSysInfo::Deepin),
       4, uint(Dtk::Core::DSysInfo::ArchLinux),
       5, uint(Dtk::Core::DSysInfo::CentOS),
       6, uint(Dtk::Core::DSysInfo::Debian),
       7, uint(Dtk::Core::DSysInfo::Fedora),
       8, uint(Dtk::Core::DSysInfo::LinuxMint),
       9, uint(Dtk::Core::DSysInfo::Manjaro),
      10, uint(Dtk::Core::DSysInfo::openSUSE),
      11, uint(Dtk::Core::DSysInfo::SailfishOS),
      12, uint(Dtk::Core::DSysInfo::Ubuntu),
      13, uint(Dtk::Core::DSysInfo::Uos),
      14, uint(Dtk::Core::DSysInfo::Gentoo),
      15, uint(Dtk::Core::DSysInfo::NixOS),
      17, uint(Dtk::Core::DSysInfo::ARM64),
      18, uint(Dtk::Core::DSysInfo::ARM64_BE),
      19, uint(Dtk::Core::DSysInfo::ARM),
      20, uint(Dtk::Core::DSysInfo::ARM_BE),
      21, uint(Dtk::Core::DSysInfo::ALPHA),
      22, uint(Dtk::Core::DSysInfo::SW_64),
      23, uint(Dtk::Core::DSysInfo::ARC),
      24, uint(Dtk::Core::DSysInfo::ARC_BE),
      25, uint(Dtk::Core::DSysInfo::CRIS),
      26, uint(Dtk::Core::DSysInfo::X86_64),
      27, uint(Dtk::Core::DSysInfo::X86),
      28, uint(Dtk::Core::DSysInfo::IA64),
      29, uint(Dtk::Core::DSysInfo::LOONGARCH64),
      30, uint(Dtk::Core::DSysInfo::M68K),
      31, uint(Dtk::Core::DSysInfo::MIPS64_LE),
      32, uint(Dtk::Core::DSysInfo::MIPS64),
      33, uint(Dtk::Core::DSysInfo::MIPS_LE),
      34, uint(Dtk::Core::DSysInfo::MIPS),
      35, uint(Dtk::Core::DSysInfo::NIOS2),
      36, uint(Dtk::Core::DSysInfo::PARISC64),
      37, uint(Dtk::Core::DSysInfo::PARISC),
      38, uint(Dtk::Core::DSysInfo::PPC64_LE),
      39, uint(Dtk::Core::DSysInfo::PPC64),
      40, uint(Dtk::Core::DSysInfo::PPC),
      41, uint(Dtk::Core::DSysInfo::PPC_LE),
      42, uint(Dtk::Core::DSysInfo::RISCV32),
      43, uint(Dtk::Core::DSysInfo::RISCV64),
      44, uint(Dtk::Core::DSysInfo::S390X),
      45, uint(Dtk::Core::DSysInfo::S390),
      46, uint(Dtk::Core::DSysInfo::SH64),
      47, uint(Dtk::Core::DSysInfo::SH),
      48, uint(Dtk::Core::DSysInfo::SPARC64),
      49, uint(Dtk::Core::DSysInfo::SPARC),
      50, uint(Dtk::Core::DSysInfo::TILEGX),
      51, uint(Dtk::Core::DSysInfo::NUM_ARCHES),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject Dtk::Core::DSysInfo::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_Dtk__Core__DSysInfo.data,
    qt_meta_data_Dtk__Core__DSysInfo,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
