/****************************************************************************
** Meta object code from reading C++ file 'ddesktopentry.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../include/global/ddesktopentry.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ddesktopentry.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk4Core13DDesktopEntryE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Core::DDesktopEntry::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk4Core13DDesktopEntryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Core::DDesktopEntry",
        "EntryType",
        "Unknown",
        "Application",
        "Link",
        "Directory",
        "ServiceType",
        "Service",
        "FSDevice",
        "ValueType",
        "Unparsed",
        "String",
        "Strings",
        "Boolean",
        "Numeric",
        "NotExisted",
        "Status",
        "NoError",
        "AccessError",
        "FormatError"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'EntryType'
        QtMocHelpers::EnumData<enum EntryType>(1, 1, QMC::EnumFlags{}).add({
            {    2, EntryType::Unknown },
            {    3, EntryType::Application },
            {    4, EntryType::Link },
            {    5, EntryType::Directory },
            {    6, EntryType::ServiceType },
            {    7, EntryType::Service },
            {    8, EntryType::FSDevice },
        }),
        // enum 'ValueType'
        QtMocHelpers::EnumData<enum ValueType>(9, 9, QMC::EnumFlags{}).add({
            {   10, ValueType::Unparsed },
            {   11, ValueType::String },
            {   12, ValueType::Strings },
            {   13, ValueType::Boolean },
            {   14, ValueType::Numeric },
            {   15, ValueType::NotExisted },
        }),
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(16, 16, QMC::EnumFlags{}).add({
            {   17, Status::NoError },
            {   18, Status::AccessError },
            {   19, Status::FormatError },
        }),
    };
    return QtMocHelpers::metaObjectData<DDesktopEntry, qt_meta_tag_ZN3Dtk4Core13DDesktopEntryE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Core::DDesktopEntry::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk4Core13DDesktopEntryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk4Core13DDesktopEntryE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk4Core13DDesktopEntryE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
