/****************************************************************************
** Meta object code from reading C++ file 'dsysinfo.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../include/global/dsysinfo.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dsysinfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3Dtk4Core8DSysInfoE_t {};
} // unnamed namespace

template <> constexpr inline auto Dtk::Core::DSysInfo::qt_create_metaobjectdata<qt_meta_tag_ZN3Dtk4Core8DSysInfoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Dtk::Core::DSysInfo",
        "ProductType",
        "UnknownType",
        "Deepin",
        "ArchLinux",
        "CentOS",
        "Debian",
        "Fedora",
        "LinuxMint",
        "Manjaro",
        "openSUSE",
        "SailfishOS",
        "Ubuntu",
        "Uos",
        "Gentoo",
        "NixOS",
        "Arch",
        "ARM64",
        "ARM64_BE",
        "ARM",
        "ARM_BE",
        "ALPHA",
        "SW_64",
        "ARC",
        "ARC_BE",
        "CRIS",
        "X86_64",
        "X86",
        "IA64",
        "LOONGARCH64",
        "M68K",
        "MIPS64_LE",
        "MIPS64",
        "MIPS_LE",
        "MIPS",
        "NIOS2",
        "PARISC64",
        "PARISC",
        "PPC64_LE",
        "PPC64",
        "PPC",
        "PPC_LE",
        "RISCV32",
        "RISCV64",
        "S390X",
        "S390",
        "SH64",
        "SH",
        "SPARC64",
        "SPARC",
        "TILEGX",
        "NUM_ARCHES"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ProductType'
        QtMocHelpers::EnumData<enum ProductType>(1, 1, QMC::EnumFlags{}).add({
            {    2, ProductType::UnknownType },
            {    3, ProductType::Deepin },
            {    4, ProductType::ArchLinux },
            {    5, ProductType::CentOS },
            {    6, ProductType::Debian },
            {    7, ProductType::Fedora },
            {    8, ProductType::LinuxMint },
            {    9, ProductType::Manjaro },
            {   10, ProductType::openSUSE },
            {   11, ProductType::SailfishOS },
            {   12, ProductType::Ubuntu },
            {   13, ProductType::Uos },
            {   14, ProductType::Gentoo },
            {   15, ProductType::NixOS },
        }),
        // enum 'Arch'
        QtMocHelpers::EnumData<enum Arch>(16, 16, QMC::EnumFlags{}).add({
            {   17, Arch::ARM64 },
            {   18, Arch::ARM64_BE },
            {   19, Arch::ARM },
            {   20, Arch::ARM_BE },
            {   21, Arch::ALPHA },
            {   22, Arch::SW_64 },
            {   23, Arch::ARC },
            {   24, Arch::ARC_BE },
            {   25, Arch::CRIS },
            {   26, Arch::X86_64 },
            {   27, Arch::X86 },
            {   28, Arch::IA64 },
            {   29, Arch::LOONGARCH64 },
            {   30, Arch::M68K },
            {   31, Arch::MIPS64_LE },
            {   32, Arch::MIPS64 },
            {   33, Arch::MIPS_LE },
            {   34, Arch::MIPS },
            {   35, Arch::NIOS2 },
            {   36, Arch::PARISC64 },
            {   37, Arch::PARISC },
            {   38, Arch::PPC64_LE },
            {   39, Arch::PPC64 },
            {   40, Arch::PPC },
            {   41, Arch::PPC_LE },
            {   42, Arch::RISCV32 },
            {   43, Arch::RISCV64 },
            {   44, Arch::S390X },
            {   45, Arch::S390 },
            {   46, Arch::SH64 },
            {   47, Arch::SH },
            {   48, Arch::SPARC64 },
            {   49, Arch::SPARC },
            {   50, Arch::TILEGX },
            {   51, Arch::NUM_ARCHES },
        }),
    };
    return QtMocHelpers::metaObjectData<DSysInfo, qt_meta_tag_ZN3Dtk4Core8DSysInfoE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Dtk::Core::DSysInfo::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk4Core8DSysInfoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3Dtk4Core8DSysInfoE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3Dtk4Core8DSysInfoE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
