/****************************************************************************
** Meta object code from reading C++ file 'manager_interface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "manager_interface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'manager_interface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16DSGConfigManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto DSGConfigManager::qt_create_metaobjectdata<qt_meta_tag_ZN16DSGConfigManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DSGConfigManager",
        "valueChanged",
        "",
        "key",
        "description",
        "QDBusPendingReply<QString>",
        "language",
        "isDefaultValue",
        "QDBusPendingReply<bool>",
        "name",
        "permissions",
        "release",
        "QDBusPendingReply<>",
        "reset",
        "setValue",
        "QDBusVariant",
        "value",
        "QDBusPendingReply<QDBusVariant>",
        "visibility",
        "keyList",
        "version"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'valueChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'description'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &, const QString &)>(4, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'isDefaultValue'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(7, 2, QMC::AccessPublic, 0x80000000 | 8, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'name'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &, const QString &)>(9, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 6 },
        }}),
        // Slot 'permissions'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &)>(10, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'release'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(11, 2, QMC::AccessPublic, 0x80000000 | 12),
        // Slot 'reset'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(13, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'setValue'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QDBusVariant &)>(14, 2, QMC::AccessPublic, 0x80000000 | 12, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'value'
        QtMocHelpers::SlotData<QDBusPendingReply<QDBusVariant>(const QString &)>(16, 2, QMC::AccessPublic, 0x80000000 | 17, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'visibility'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>(const QString &)>(18, 2, QMC::AccessPublic, 0x80000000 | 5, {{
            { QMetaType::QString, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'keyList'
        QtMocHelpers::PropertyData<QStringList>(19, QMetaType::QStringList, QMC::DefaultPropertyFlags),
        // property 'version'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DSGConfigManager, qt_meta_tag_ZN16DSGConfigManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DSGConfigManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DSGConfigManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DSGConfigManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16DSGConfigManagerE_t>.metaTypes,
    nullptr
} };

void DSGConfigManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DSGConfigManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->valueChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: { QDBusPendingReply<QString> _r = _t->description((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 2: { QDBusPendingReply<bool> _r = _t->isDefaultValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 3: { QDBusPendingReply<QString> _r = _t->name((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 4: { QDBusPendingReply<QString> _r = _t->permissions((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<> _r = _t->release();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<> _r = _t->reset((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->setValue((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QDBusVariant>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<QDBusVariant> _r = _t->value((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QDBusVariant>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<QString> _r = _t->visibility((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DSGConfigManager::*)(const QString & )>(_a, &DSGConfigManager::valueChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->keyList(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->version(); break;
        default: break;
        }
    }
}

const QMetaObject *DSGConfigManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DSGConfigManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DSGConfigManagerE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int DSGConfigManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void DSGConfigManager::valueChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
