/****************************************************************************
** Meta object code from reading C++ file 'ddesktopentry.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../include/global/ddesktopentry.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'ddesktopentry.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Dtk__Core__DDesktopEntry_t {
    QByteArrayData data[20];
    char stringdata0[199];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Dtk__Core__DDesktopEntry_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Dtk__Core__DDesktopEntry_t qt_meta_stringdata_Dtk__Core__DDesktopEntry = {
    {
QT_MOC_LITERAL(0, 0, 24), // "Dtk::Core::DDesktopEntry"
QT_MOC_LITERAL(1, 25, 9), // "EntryType"
QT_MOC_LITERAL(2, 35, 7), // "Unknown"
QT_MOC_LITERAL(3, 43, 11), // "Application"
QT_MOC_LITERAL(4, 55, 4), // "Link"
QT_MOC_LITERAL(5, 60, 9), // "Directory"
QT_MOC_LITERAL(6, 70, 11), // "ServiceType"
QT_MOC_LITERAL(7, 82, 7), // "Service"
QT_MOC_LITERAL(8, 90, 8), // "FSDevice"
QT_MOC_LITERAL(9, 99, 9), // "ValueType"
QT_MOC_LITERAL(10, 109, 8), // "Unparsed"
QT_MOC_LITERAL(11, 118, 6), // "String"
QT_MOC_LITERAL(12, 125, 7), // "Strings"
QT_MOC_LITERAL(13, 133, 7), // "Boolean"
QT_MOC_LITERAL(14, 141, 7), // "Numeric"
QT_MOC_LITERAL(15, 149, 10), // "NotExisted"
QT_MOC_LITERAL(16, 160, 6), // "Status"
QT_MOC_LITERAL(17, 167, 7), // "NoError"
QT_MOC_LITERAL(18, 175, 11), // "AccessError"
QT_MOC_LITERAL(19, 187, 11) // "FormatError"

    },
    "Dtk::Core::DDesktopEntry\0EntryType\0"
    "Unknown\0Application\0Link\0Directory\0"
    "ServiceType\0Service\0FSDevice\0ValueType\0"
    "Unparsed\0String\0Strings\0Boolean\0Numeric\0"
    "NotExisted\0Status\0NoError\0AccessError\0"
    "FormatError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Dtk__Core__DDesktopEntry[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       3,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,    7,   29,
       9,    9, 0x0,    6,   43,
      16,   16, 0x0,    3,   55,

 // enum data: key, value
       2, uint(Dtk::Core::DDesktopEntry::Unknown),
       3, uint(Dtk::Core::DDesktopEntry::Application),
       4, uint(Dtk::Core::DDesktopEntry::Link),
       5, uint(Dtk::Core::DDesktopEntry::Directory),
       6, uint(Dtk::Core::DDesktopEntry::ServiceType),
       7, uint(Dtk::Core::DDesktopEntry::Service),
       8, uint(Dtk::Core::DDesktopEntry::FSDevice),
      10, uint(Dtk::Core::DDesktopEntry::Unparsed),
      11, uint(Dtk::Core::DDesktopEntry::String),
      12, uint(Dtk::Core::DDesktopEntry::Strings),
      13, uint(Dtk::Core::DDesktopEntry::Boolean),
      14, uint(Dtk::Core::DDesktopEntry::Numeric),
      15, uint(Dtk::Core::DDesktopEntry::NotExisted),
      17, uint(Dtk::Core::DDesktopEntry::NoError),
      18, uint(Dtk::Core::DDesktopEntry::AccessError),
      19, uint(Dtk::Core::DDesktopEntry::FormatError),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject Dtk::Core::DDesktopEntry::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_Dtk__Core__DDesktopEntry.data,
    qt_meta_data_Dtk__Core__DDesktopEntry,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
